# -*- coding: utf-8 -*-
"""This file contains the Windows NT Known Folder identifier definitions."""

# For now ignore the line too long errors.
# pylint: disable=line-too-long

# For now copied from:
# https://code.google.com/p/libfwsi/wiki/KnownFolderIdentifiers

# TODO: store these in a database or equiv.

DESCRIPTIONS = {
    u'008ca0b1-55b4-4c56-b8a8-4de4b299d3be': u'Account Pictures',
    u'00bcfc5a-ed94-4e48-96a1-3f6217f21990': u'Roaming Tiles',
    u'0139d44e-6afe-49f2-8690-3dafcae6ffb8': u'(Common) Programs',
    u'0482af6c-08f1-4c34-8c90-e17ec98b1e17': u'Public Account Pictures',
    u'054fae61-4dd8-4787-80b6-090220c4b700': u'Game Explorer (Game Tasks)',
    u'0762d272-c50a-4bb0-a382-697dcd729b80': u'Users (User Profiles)',
    u'0ac0837c-bbf8-452a-850d-79d08e667ca7': u'Computer (My Computer)',
    u'0d4c3db6-03a3-462f-a0e6-08924c41b5d4': u'History',
    u'0f214138-b1d3-4a90-bba9-27cbc0c5389a': u'Sync Setup',
    u'15ca69b3-30ee-49c1-ace1-6b5ec372afb5': u'Sample Playlists',
    u'1777f761-68ad-4d8a-87bd-30b759fa33dd': u'Favorites',
    u'18989b1d-99b5-455b-841c-ab7c74e4ddfc': u'Videos (My Video)',
    u'190337d1-b8ca-4121-a639-6d472d16972a': u'Search Results (Search Home)',
    u'1a6fdba2-f42d-4358-a798-b74d745926c5': u'Recorded TV',
    u'1ac14e77-02e7-4e5d-b744-2eb1ae5198b7': u'System32 (System)',
    u'1b3ea5dc-b587-4786-b4ef-bd1dc332aeae': u'Libraries',
    u'1e87508d-89c2-42f0-8a7e-645a0f50ca58': u'Applications',
    u'2112ab0a-c86a-4ffe-a368-0de96e47012e': u'Music',
    u'2400183a-6185-49fb-a2d8-4a392a602ba3': u'Public Videos (Common Video)',
    u'24d89e24-2f19-4534-9dde-6a6671fbb8fe': u'One Drive Documents',
    u'289a9a43-be44-4057-a41b-587a76d7e7f9': u'Sync Results',
    u'2a00375e-224c-49de-b8d1-440df7ef3ddc': u'Localized Resources (Directory)',
    u'2b0f765d-c0e9-4171-908e-08a611b84ff6': u'Cookies',
    u'2c36c0aa-5812-4b87-bfd0-4cd0dfb19b39': u'Original Images',
    u'3214fab5-9757-4298-bb61-92a9deaa44ff': u'Public Music (Common Music)',
    u'339719b5-8c47-4894-94c2-d8f77add44a6': u'One Drive Pictures',
    u'33e28130-4e1e-4676-835a-98395c3bc3bb': u'Pictures (My Pictures)',
    u'352481e8-33be-4251-ba85-6007caedcf9d': u'Internet Cache (Temporary Internet Files)',
    u'374de290-123f-4565-9164-39c4925e467b': u'Downloads',
    u'3d644c9b-1fb8-4f30-9b45-f670235f79c0': u'Public Downloads (Common Downloads)',
    u'3eb685db-65f9-4cf6-a03a-e3ef65729f3d': u'Roaming Application Data (Roaming)',
    u'43668bf8-c14e-49b2-97c9-747784d784b7': u'Sync Center (Sync Manager)',
    u'48daf80b-e6cf-4f4e-b800-0e69d84ee384': u'Libraries',
    u'491e922f-5643-4af4-a7eb-4e7a138d8174': u'Videos',
    u'4bd8d571-6d19-48d3-be97-422220080e43': u'Music (My Music)',
    u'4bfefb45-347d-4006-a5be-ac0cb0567192': u'Conflicts',
    u'4c5c32ff-bb9d-43b0-b5b4-2d72e54eaaa4': u'Saved Games',
    u'4d9f7874-4e0c-4904-967b-40b0d20c3e4b': u'Internet (The Internet)',
    u'52528a6b-b9e3-4add-b60d-588c2dba842d': u'Homegroup',
    u'52a4f021-7b75-48a9-9f6b-4b87a210bc8f': u'Quick Launch',
    u'56784854-c6cb-462b-8169-88e350acb882': u'Contacts',
    u'5b3749ad-b49f-49c1-83eb-15370fbd4882': u'Tree Properties',
    u'5cd7aee2-2219-4a67-b85d-6c9ce15660cb': u'Programs',
    u'5ce4a5e9-e4eb-479d-b89f-130c02886155': u'Device Metadata Store',
    u'5e6c858f-0e22-4760-9afe-ea3317b67173': u'Profile (User\'s name)',
    u'625b53c3-ab48-4ec1-ba1f-a1ef4146fc19': u'Start Menu',
    u'62ab5d82-fdc1-4dc3-a9dd-070d1d495d97': u'Program Data',
    u'6365d5a7-0f0d-45e5-87f6-0da56b6a4f7d': u'Common Files (x64)',
    u'69d2cf90-fc33-4fb7-9a0c-ebb0f0fcb43c': u'Slide Shows (Photo Albums)',
    u'6d809377-6af0-444b-8957-a3773f02200e': u'Program Files (x64)',
    u'6f0cd92b-2e97-45d1-88ff-b0d186b8dedd': u'Network Connections',
    u'724ef170-a42d-4fef-9f26-b60e846fba4f': u'Administrative Tools',
    u'767e6811-49cb-4273-87c2-20f355e1085b': u'One Drive Camera Roll',
    u'76fc4e2d-d6ad-4519-a663-37bd56068185': u'Printers',
    u'7b0db17d-9cd2-4a93-9733-46cc89022e7c': u'Documents',
    u'7b396e54-9ec5-4300-be0a-2482ebae1a26': u'Default Gadgets (Sidebar Default Parts)',
    u'7c5a40ef-a0fb-4bfc-874a-c0f2e0b9fa8e': u'Program Files (x86)',
    u'7d1d3a04-debb-4115-95cf-2f29da2920da': u'Saved Searches (Searches)',
    u'7e636bfe-dfa9-4d5e-b456-d7b39851d8a9': u'Templates',
    u'82a5ea35-d9cd-47c5-9629-e15d2f714e6e': u'(Common) Startup',
    u'82a74aeb-aeb4-465c-a014-d097ee346d63': u'Control Panel',
    u'859ead94-2e85-48ad-a71a-0969cb56a6cd': u'Sample Videos',
    u'8983036c-27c0-404b-8f08-102d10dcfd74': u'Send To',
    u'8ad10c31-2adb-4296-a8f7-e4701232c972': u'Resources (Resources Directory)',
    u'905e63b6-c1bf-494e-b29c-65b732d3d21a': u'Program Files',
    u'9274bd8d-cfd1-41c3-b35e-b13f55a758f4': u'Printer Shortcuts (PrintHood)',
    u'98ec0e18-2098-4d44-8644-66979315a281': u'Microsoft Office Outlook (MAPI)',
    u'9b74b6a3-0dfd-4f11-9e78-5f7800f2e772': u'User\'s name',
    u'9e3995ab-1f9c-4f13-b827-48b24b6c7174': u'User Pinned',
    u'9e52ab10-f80d-49df-acb8-4330f5687855': u'Temporary Burn Folder (CD Burning)',
    u'a302545d-deff-464b-abe8-61c8648d939b': u'Libraries',
    u'a305ce99-f527-492b-8b1a-7e76fa98d6e4': u'Installed Updates (Application Updates)',
    u'a3918781-e5f2-4890-b3d9-a7e54332328c': u'Application Shortcuts',
    u'a4115719-d62e-491d-aa7c-e74b8be3b067': u'(Common) Start Menu',
    u'a520a1a4-1780-4ff6-bd18-167343c5af16': u'Local Application Data Low (Local Low)',
    u'a52bba46-e9e1-435f-b3d9-28daa648c0f6': u'One Drive',
    u'a63293e8-664e-48db-a079-df759e0509f7': u'Templates',
    u'a75d362e-50fc-4fb7-ac2c-a8beaa314493': u'Gadgets (Sidebar Parts)',
    u'a77f5d77-2e2b-44c3-a6a2-aba601054a51': u'Programs',
    u'a990ae9f-a03b-4e80-94bc-9912d7504104': u'Pictures',
    u'aaa8d5a5-f1d6-4259-baa8-78e7ef60835e': u'Roamed Tile Images',
    u'ab5fb87b-7ce2-4f83-915d-550846c9537b': u'Camera Roll',
    u'ae50c081-ebd2-438a-8655-8a092e34987a': u'Recent (Recent Items)',
    u'b250c668-f57d-4ee1-a63c-290ee7d1aa1f': u'Sample Music',
    u'b4bfcc3a-db2c-424c-b029-7fe99a87c641': u'Desktop',
    u'b6ebfb86-6907-413c-9af7-4fc2abf07cc5': u'Public Pictures (Common Pictures)',
    u'b7534046-3ecb-4c18-be4e-64cd4cb7d6ac': u'Recycle Bin (Bit Bucket)',
    u'b7bede81-df94-4682-a7d8-57a52620b86f': u'Screenshots',
    u'b94237e7-57ac-4347-9151-b08c6c32d1f7': u'(Common) Templates',
    u'b97d20bb-f46a-4c97-ba10-5e3608430854': u'Startup',
    u'bcb5256f-79f6-4cee-b725-dc34e402fd46': u'Implicit Application Shortcuts',
    u'bcbd3057-ca5c-4622-b42d-bc56db0ae516': u'Programs',
    u'bd85e001-112e-431e-983b-7b15ac09fff1': u'Recorded TV',
    u'bfb9d5e0-c6a9-404c-b2b2-ae6db6af4968': u'Links',
    u'c1bae2d0-10df-4334-bedd-7aa20b227a9d': u'(Common) OEM Links',
    u'c4900540-2379-4c75-844b-64e6faf8716b': u'Sample Pictures',
    u'c4aa340d-f20f-4863-afef-f87ef2e6ba25': u'Public Desktop (Common Desktop)',
    u'c5abbf53-e17f-4121-8900-86626fc2c973': u'Network Shortcuts (NetHood)',
    u'c870044b-f49e-4126-a9c3-b52a1ff411e8': u'Ringtones',
    u'cac52c1a-b53d-4edc-92d7-6b2e8ac19434': u'Games',
    u'd0384e7d-bac3-4797-8f14-cba229b392b5': u'(Common) Administrative Tools',
    u'd20beec4-5ca8-4905-ae3b-bf251ea09b53': u'Network (Places)',
    u'd65231b0-b2f1-4857-a4ce-a8e7c6ea7d27': u'System32 (x86)',
    u'd9dc8a3b-b784-432e-a781-5a1130a75963': u'History',
    u'de61d971-5ebc-4f02-a3a9-6c82895e5c04': u'Add New Programs (Get Programs)',
    u'de92c1c7-837f-4f69-a3bb-86e631204a23': u'Playlists',
    u'de974d24-d9c6-4d3e-bf91-f4455120b917': u'Common Files (x86)',
    u'debf2536-e1a8-4c59-b6a2-414586476aea': u'Game Explorer (Public Game Tasks)',
    u'df7266ac-9274-4867-8d55-3bd661de872d': u'Programs and Features (Change and Remove Programs)',
    u'dfdf76a2-c82a-4d63-906a-5644ac457385': u'Public',
    u'e555ab60-153b-4d17-9f04-a5fe99fc15ec': u'Ringtones',
    u'ed4824af-dce4-45a8-81e2-fc7965083634': u'Public Documents (Common Documents)',
    u'ee32e446-31ca-4aba-814f-a5ebd2fd6d5e': u'Offline Files (CSC)',
    u'f1b32785-6fba-4fcf-9d55-7b8e7f157091': u'Local Application Data',
    u'f38bf404-1d43-42f2-9305-67de0b28fc23': u'Windows',
    u'f3ce0f7c-4901-4acc-8648-d5d44b04ef8f': u'User\'s Files',
    u'f7f1ed05-9f6d-47a2-aaae-29d317c6f066': u'Common Files',
    u'fd228cb7-ae11-4ae3-864c-16f3910ab8fe': u'Fonts',
    u'fdd39ad0-238f-46af-adb4-6c85480369c7': u'Documents (Personal)',
}

PATHS = {
    u'008ca0b1-55b4-4c56-b8a8-4de4b299d3be': u'%APPDATA%\\Microsoft\\Windows\\AccountPictures',
    u'00bcfc5a-ed94-4e48-96a1-3f6217f21990': u'%LOCALAPPDATA%\\Microsoft\\Windows\\RoamingTiles',
    u'0139d44e-6afe-49f2-8690-3dafcae6ffb8': u'%ALLUSERSPROFILE%\\Microsoft\\Windows\\Start Menu\\Programs',
    u'0482af6c-08f1-4c34-8c90-e17ec98b1e17': u'%PUBLIC%\\AccountPictures',
    u'054fae61-4dd8-4787-80b6-090220c4b700': u'%LOCALAPPDATA%\\Microsoft\\Windows\\GameExplorer',
    u'0762d272-c50a-4bb0-a382-697dcd729b80': u'%SYSTEMDRIVE%\\Users',
    u'0ac0837c-bbf8-452a-850d-79d08e667ca7': u'',
    u'0d4c3db6-03a3-462f-a0e6-08924c41b5d4': u'%LOCALAPPDATA%\\Microsoft\\Windows\\ConnectedSearch\\History',
    u'0f214138-b1d3-4a90-bba9-27cbc0c5389a': u'',
    u'15ca69b3-30ee-49c1-ace1-6b5ec372afb5': u'%PUBLIC%\\Music\\Sample Playlists',
    u'1777f761-68ad-4d8a-87bd-30b759fa33dd': u'%USERPROFILE%\\Favorites',
    u'18989b1d-99b5-455b-841c-ab7c74e4ddfc': u'%USERPROFILE%\\Videos',
    u'190337d1-b8ca-4121-a639-6d472d16972a': u'',
    u'1a6fdba2-f42d-4358-a798-b74d745926c5': u'%PUBLIC%\\RecordedTV.library-ms',
    u'1ac14e77-02e7-4e5d-b744-2eb1ae5198b7': u'%WINDIR%\\System32',
    u'1b3ea5dc-b587-4786-b4ef-bd1dc332aeae': u'%APPDATA%\\Microsoft\\Windows\\Libraries',
    u'1e87508d-89c2-42f0-8a7e-645a0f50ca58': u'',
    u'2112ab0a-c86a-4ffe-a368-0de96e47012e': u'%APPDATA%\\Microsoft\\Windows\\Libraries\\Music.library-ms',
    u'2400183a-6185-49fb-a2d8-4a392a602ba3': u'%PUBLIC%\\Videos',
    u'24d89e24-2f19-4534-9dde-6a6671fbb8fe': u'%USERPROFILE%\\OneDrive\\Documents',
    u'289a9a43-be44-4057-a41b-587a76d7e7f9': u'',
    u'2a00375e-224c-49de-b8d1-440df7ef3ddc': u'%WINDIR%\\resources\\%CODEPAGE%',
    u'2b0f765d-c0e9-4171-908e-08a611b84ff6': u'%APPDATA%\\Microsoft\\Windows\\Cookies',
    u'2c36c0aa-5812-4b87-bfd0-4cd0dfb19b39': u'%LOCALAPPDATA%\\Microsoft\\Windows Photo Gallery\\Original Images',
    u'3214fab5-9757-4298-bb61-92a9deaa44ff': u'%PUBLIC%\\Music',
    u'339719b5-8c47-4894-94c2-d8f77add44a6': u'%USERPROFILE%\\OneDrive\\Pictures',
    u'33e28130-4e1e-4676-835a-98395c3bc3bb': u'%USERPROFILE%\\Pictures',
    u'352481e8-33be-4251-ba85-6007caedcf9d': u'%LOCALAPPDATA%\\Microsoft\\Windows\\Temporary Internet Files',
    u'374de290-123f-4565-9164-39c4925e467b': u'%USERPROFILE%\\Downloads',
    u'3d644c9b-1fb8-4f30-9b45-f670235f79c0': u'%PUBLIC%\\Downloads',
    u'3eb685db-65f9-4cf6-a03a-e3ef65729f3d': u'%USERPROFILE%\\AppData\\Roaming',
    u'43668bf8-c14e-49b2-97c9-747784d784b7': u'',
    u'48daf80b-e6cf-4f4e-b800-0e69d84ee384': u'%ALLUSERSPROFILE%\\Microsoft\\Windows\\Libraries',
    u'491e922f-5643-4af4-a7eb-4e7a138d8174': u'%APPDATA%\\Microsoft\\Windows\\Libraries\\Videos.library-ms',
    u'4bd8d571-6d19-48d3-be97-422220080e43': u'%USERPROFILE%\\Music',
    u'4bfefb45-347d-4006-a5be-ac0cb0567192': u'',
    u'4c5c32ff-bb9d-43b0-b5b4-2d72e54eaaa4': u'%USERPROFILE%\\Saved Games',
    u'4d9f7874-4e0c-4904-967b-40b0d20c3e4b': u'',
    u'52528a6b-b9e3-4add-b60d-588c2dba842d': u'',
    u'52a4f021-7b75-48a9-9f6b-4b87a210bc8f': u'%APPDATA%\\Microsoft\\Internet Explorer\\Quick Launch',
    u'56784854-c6cb-462b-8169-88e350acb882': u'',
    u'5b3749ad-b49f-49c1-83eb-15370fbd4882': u'',
    u'5cd7aee2-2219-4a67-b85d-6c9ce15660cb': u'%LOCALAPPDATA%\\Programs',
    u'5ce4a5e9-e4eb-479d-b89f-130c02886155': u'%ALLUSERSPROFILE%\\Microsoft\\Windows\\DeviceMetadataStore',
    u'5e6c858f-0e22-4760-9afe-ea3317b67173': u'%SYSTEMDRIVE%\\Users\\%USERNAME%',
    u'625b53c3-ab48-4ec1-ba1f-a1ef4146fc19': u'%APPDATA%\\Microsoft\\Windows\\Start Menu',
    u'62ab5d82-fdc1-4dc3-a9dd-070d1d495d97': u'%SYSTEMDRIVE%\\ProgramData',
    u'6365d5a7-0f0d-45e5-87f6-0da56b6a4f7d': u'%PROGRAMFILES%\\Common Files',
    u'69d2cf90-fc33-4fb7-9a0c-ebb0f0fcb43c': u'%USERPROFILE%\\Pictures\\Slide Shows',
    u'6d809377-6af0-444b-8957-a3773f02200e': u'%SYSTEMDRIVE%\\Program Files',
    u'6f0cd92b-2e97-45d1-88ff-b0d186b8dedd': u'',
    u'724ef170-a42d-4fef-9f26-b60e846fba4f': u'%APPDATA%\\Microsoft\\Windows\\Start Menu\\Programs\\Administrative Tools',
    u'767e6811-49cb-4273-87c2-20f355e1085b': u'%USERPROFILE%\\OneDrive\\Pictures\\Camera Roll',
    u'76fc4e2d-d6ad-4519-a663-37bd56068185': u'',
    u'7b0db17d-9cd2-4a93-9733-46cc89022e7c': u'%APPDATA%\\Microsoft\\Windows\\Libraries\\Documents.library-ms',
    u'7b396e54-9ec5-4300-be0a-2482ebae1a26': u'%PROGRAMFILES%\\Windows Sidebar\\Gadgets',
    u'7c5a40ef-a0fb-4bfc-874a-c0f2e0b9fa8e': u'%PROGRAMFILES% (%SYSTEMDRIVE%\\Program Files)',
    u'7d1d3a04-debb-4115-95cf-2f29da2920da': u'%USERPROFILE%\\Searches',
    u'7e636bfe-dfa9-4d5e-b456-d7b39851d8a9': u'%LOCALAPPDATA%\\Microsoft\\Windows\\ConnectedSearch\\Templates',
    u'82a5ea35-d9cd-47c5-9629-e15d2f714e6e': u'%ALLUSERSPROFILE%\\Microsoft\\Windows\\Start Menu\\Programs\\StartUp',
    u'82a74aeb-aeb4-465c-a014-d097ee346d63': u'',
    u'859ead94-2e85-48ad-a71a-0969cb56a6cd': u'%PUBLIC%\\Videos\\Sample Videos',
    u'8983036c-27c0-404b-8f08-102d10dcfd74': u'%APPDATA%\\Microsoft\\Windows\\SendTo',
    u'8ad10c31-2adb-4296-a8f7-e4701232c972': u'%WINDIR%\\Resources',
    u'905e63b6-c1bf-494e-b29c-65b732d3d21a': u'%SYSTEMDRIVE%\\Program Files',
    u'9274bd8d-cfd1-41c3-b35e-b13f55a758f4': u'%APPDATA%\\Microsoft\\Windows\\Printer Shortcuts',
    u'98ec0e18-2098-4d44-8644-66979315a281': u'',
    u'9b74b6a3-0dfd-4f11-9e78-5f7800f2e772': u'',
    u'9e3995ab-1f9c-4f13-b827-48b24b6c7174': u'%APPDATA%\\Microsoft\\Internet Explorer\\Quick Launch\\User Pinned',
    u'9e52ab10-f80d-49df-acb8-4330f5687855': u'%LOCALAPPDATA%\\Microsoft\\Windows\\Burn\\Burn',
    u'a302545d-deff-464b-abe8-61c8648d939b': u'',
    u'a305ce99-f527-492b-8b1a-7e76fa98d6e4': u'',
    u'a3918781-e5f2-4890-b3d9-a7e54332328c': u'%LOCALAPPDATA%\\Microsoft\\Windows\\Application Shortcuts',
    u'a4115719-d62e-491d-aa7c-e74b8be3b067': u'%ALLUSERSPROFILE%\\Microsoft\\Windows\\Start Menu',
    u'a520a1a4-1780-4ff6-bd18-167343c5af16': u'%USERPROFILE%\\AppData\\LocalLow',
    u'a52bba46-e9e1-435f-b3d9-28daa648c0f6': u'%USERPROFILE%\\OneDrive',
    u'a63293e8-664e-48db-a079-df759e0509f7': u'%APPDATA%\\Microsoft\\Windows\\Templates',
    u'a75d362e-50fc-4fb7-ac2c-a8beaa314493': u'%LOCALAPPDATA%\\Microsoft\\Windows Sidebar\\Gadgets',
    u'a77f5d77-2e2b-44c3-a6a2-aba601054a51': u'%APPDATA%\\Microsoft\\Windows\\Start Menu\\Programs',
    u'a990ae9f-a03b-4e80-94bc-9912d7504104': u'%APPDATA%\\Microsoft\\Windows\\Libraries\\Pictures.library-ms',
    u'aaa8d5a5-f1d6-4259-baa8-78e7ef60835e': u'%LOCALAPPDATA%\\Microsoft\\Windows\\RoamedTileImages',
    u'ab5fb87b-7ce2-4f83-915d-550846c9537b': u'%USERPROFILE%\\Pictures\\Camera Roll',
    u'ae50c081-ebd2-438a-8655-8a092e34987a': u'%APPDATA%\\Microsoft\\Windows\\Recent',
    u'b250c668-f57d-4ee1-a63c-290ee7d1aa1f': u'%PUBLIC%\\Music\\Sample Music',
    u'b4bfcc3a-db2c-424c-b029-7fe99a87c641': u'%USERPROFILE%\\Desktop',
    u'b6ebfb86-6907-413c-9af7-4fc2abf07cc5': u'%PUBLIC%\\Pictures',
    u'b7534046-3ecb-4c18-be4e-64cd4cb7d6ac': u'',
    u'b7bede81-df94-4682-a7d8-57a52620b86f': u'%USERPROFILE%\\Pictures\\Screenshots',
    u'b94237e7-57ac-4347-9151-b08c6c32d1f7': u'%ALLUSERSPROFILE%\\Microsoft\\Windows\\Templates',
    u'b97d20bb-f46a-4c97-ba10-5e3608430854': u'%APPDATA%\\Microsoft\\Windows\\Start Menu\\Programs\\StartUp',
    u'bcb5256f-79f6-4cee-b725-dc34e402fd46': u'%APPDATA%\\Microsoft\\Internet Explorer\\Quick Launch\\User Pinned\\ImplicitAppShortcuts',
    u'bcbd3057-ca5c-4622-b42d-bc56db0ae516': u'%LOCALAPPDATA%\\Programs\\Common',
    u'bd85e001-112e-431e-983b-7b15ac09fff1': u'',
    u'bfb9d5e0-c6a9-404c-b2b2-ae6db6af4968': u'%USERPROFILE%\\Links',
    u'c1bae2d0-10df-4334-bedd-7aa20b227a9d': u'%ALLUSERSPROFILE%\\OEM Links',
    u'c4900540-2379-4c75-844b-64e6faf8716b': u'%PUBLIC%\\Pictures\\Sample Pictures',
    u'c4aa340d-f20f-4863-afef-f87ef2e6ba25': u'%PUBLIC%\\Desktop',
    u'c5abbf53-e17f-4121-8900-86626fc2c973': u'%APPDATA%\\Microsoft\\Windows\\Network Shortcuts',
    u'c870044b-f49e-4126-a9c3-b52a1ff411e8': u'%LOCALAPPDATA%\\Microsoft\\Windows\\Ringtones',
    u'cac52c1a-b53d-4edc-92d7-6b2e8ac19434': u'',
    u'd0384e7d-bac3-4797-8f14-cba229b392b5': u'%ALLUSERSPROFILE%\\Microsoft\\Windows\\Start Menu\\Programs\\Administrative Tools',
    u'd20beec4-5ca8-4905-ae3b-bf251ea09b53': u'',
    u'd65231b0-b2f1-4857-a4ce-a8e7c6ea7d27': u'%WINDIR%\\system32',
    u'd9dc8a3b-b784-432e-a781-5a1130a75963': u'%LOCALAPPDATA%\\Microsoft\\Windows\\History',
    u'de61d971-5ebc-4f02-a3a9-6c82895e5c04': u'',
    u'de92c1c7-837f-4f69-a3bb-86e631204a23': u'%USERPROFILE%\\Music\\Playlists',
    u'de974d24-d9c6-4d3e-bf91-f4455120b917': u'%PROGRAMFILES%\\Common Files',
    u'debf2536-e1a8-4c59-b6a2-414586476aea': u'%ALLUSERSPROFILE%\\Microsoft\\Windows\\GameExplorer',
    u'df7266ac-9274-4867-8d55-3bd661de872d': u'',
    u'dfdf76a2-c82a-4d63-906a-5644ac457385': u'%SYSTEMDRIVE%\\Users\\Public',
    u'e555ab60-153b-4d17-9f04-a5fe99fc15ec': u'%ALLUSERSPROFILE%\\Microsoft\\Windows\\Ringtones',
    u'ed4824af-dce4-45a8-81e2-fc7965083634': u'%PUBLIC%\\Documents',
    u'ee32e446-31ca-4aba-814f-a5ebd2fd6d5e': u'',
    u'f1b32785-6fba-4fcf-9d55-7b8e7f157091': u'%USERPROFILE%\\AppData\\Local',
    u'f38bf404-1d43-42f2-9305-67de0b28fc23': u'%WINDIR%',
    u'f3ce0f7c-4901-4acc-8648-d5d44b04ef8f': u'',
    u'f7f1ed05-9f6d-47a2-aaae-29d317c6f066': u'%PROGRAMFILES%\\Common Files',
    u'fd228cb7-ae11-4ae3-864c-16f3910ab8fe': u'%WINDIR%\\Fonts',
    u'fdd39ad0-238f-46af-adb4-6c85480369c7': u'%USERPROFILE%\\Documents',
}
