# -*- coding: utf-8 -*-
"""This file contains the definitions for analysis plugins."""

# All the possible analysis plugin types.

# Plugin that detects anomalies.
PLUGIN_TYPE_ANOMALY = 1

# Plugin that calculating statistical information.
PLUGIN_TYPE_STATISTICS = 2

# Plugin that annotates (or tags) event objects.
PLUGIN_TYPE_ANNOTATION = 3

# Plugin that provides summary information.
PLUGIN_TYPE_REPORT = 4
