#!/usr/bin/python
# -*- coding: utf-8 -*-
"""Tests the multi-process processing engine."""

import unittest

from tests import test_lib as shared_test_lib


class MultiProcessEngineTest(shared_test_lib.BaseTestCase):
  """Tests for the multi-process engine."""

  # TODO: add test for _AbortJoin
  # TODO: add test for _AbortKill
  # TODO: add test for _AbortTerminate
  # TODO: add test for _CheckStatusWorkerProcess
  # TODO: add test for _KillProcess
  # TODO: add test for _LogMemoryUsage
  # TODO: add test for _ProfilingSampleMemory
  # TODO: add test for _QueryProcessStatus
  # TODO: add test for _RaiseIfNotMonitored
  # TODO: add test for _RaiseIfNotRegistered
  # TODO: add test for _RegisterProcess
  # TODO: add test for _StartMonitoringProcess
  # TODO: add test for _StartStatusUpdateThread
  # TODO: add test for _StopMonitoringProcess
  # TODO: add test for _StopMonitoringProcesses
  # TODO: add test for _StopStatusUpdateThread
  # TODO: add test for _TerminateProcess


if __name__ == '__main__':
  unittest.main()
