# -*- coding: utf-8 -*-
"""Browser cookie plugin related functions and classes for testing."""

from tests.parsers import test_lib


class CookiePluginTestCase(test_lib.ParserTestCase):
  """The unit test case for a browser cookie plugin."""
